#' Create a skeleton for a new source package
#'
#' @param packageName name of the package (and).
#' @param build.dir   name of directory to dump built package in (default
#' 'bin').
#' @param hg          boolean, whether to set this up as a mercurial
#' project (initialises the project in mercurial)
#' @param hg.commit   whether to do an initial commit of the mercurial repo
#' @param empty       boolean, whether to create an entirely empty package
#' (ie ignores list and env arguments).
#' @param path        path that the entire project directory will be *placed in*.
#' @param force       boolean, whether to *REMOVE AND WRITE OVER* existing
#'                    directories
#' @param ... passed to \code{\link[utils]{package.skeleton}}
#' @details
#' The difference between this and the base
#' \code{\link[utils]{package.skeleton}} is that this sets up the project
#' like so:
#'
#' \preformatted{
#' path
#' |- packageName-package     The "project" directory
#'    |- Makefile     A makefile for compiling various bits
#'    |- README       A readme for the developer
#'    |- build.dir    The output directory things are built in
#'    |- packageName  The actual R package directory
#'       |- R  
#'       |- inst      ... etc (packages created by R's built-in
#'       |- man                'package.skeleton' function).
#' }
#'
#' The in-built \code{\link[utils]{package.skeleton}} function only creates
#' the internal 'packageName' directory.
#'
#' This version sets up the package in an environment more suited to a
#' developer, with a handy Makefile to generate documentation ('make doc'),
#' generate the package ('make' or 'make package'), check the package
#' ('make check'), and/or install it ('make install').
#'
#' It also separates source code (in the packageName dir) from the build
#' and check outputs (in the build.dir dir).
#'
#' If the mercurial versioning system is installed on the system, it can
#' optionally create a mercurial repository for the project, with a
#' pre-generated \code{.hgignore} file which ignores generated files like
#' everything in build.dir, Rd files, etc.
#'
#' The Makefile contains the targets 'doc' (to build documentation using
#' the roxygen2 package), 'package' (to build the package using \code{R CMD
#' BUILD}), 'check' (to check the package with \code{R CMD CHECK}), and
#' 'install' (to install the package with \code{R CMD INSTALL}).
#'
#' @export
my.package.skeleton <- function(packageName, path=".", build.dir='bin', 
                                hg=FALSE, hg.commit=hg, force=FALSE, 
                                empty=FALSE, ...)
{
    safe.dir.create <- function(path) {
        dirTest <- function(x) !is.na(isdir <- file.info(x)$isdir) & 
            isdir
            if (!dirTest(path) && !dir.create(path)) 
                stop(gettextf("cannot create directory '%s'",path),
                                     domain = NA)
    }

    dir <- file.path(path,sprintf('%s-package',packageName))
    if ( file.exists(dir) )
        if ( force )
            unlink(dir,recursive=T)
        else
            stop(sprintf("Directory '%s' already exists",dir))

    message("Creating project directory...")
    safe.dir.create(dir)
tryCatch({
    message("Creating build directory...")
    # make build dir
    safe.dir.create(file.path(dir,build.dir))

    # make package in there
    # NOTE: there's a bug if you have nothing in your
    #  environment when you create the package:
    # it will say "!have: invalid argument type".
    # Making something (that doesn't get included anyway) is the
    # workaround.
    DELETE_LATER <- FALSE
    if ( !is.na(i<-match('env', names(list(...)))) )
        env <- list(...)[[i]]
    else
        if ( empty )
            env <- new.env()
        else
            env <- globalenv()

    if ( (DELETE_LATER <- (length(ls(env))==0) ) ) 
        assign('.foo',1,envir=env)
    message("Creating R package directory...")
    suppressMessages(package.skeleton(packageName,path=dir,env=env,...))
    if ( DELETE_LATER ) {
        # there was nothing there anyway since the environment was blank,
        # so it's safe to remove everything:
        file.remove(list.files(file.path(dir,packageName,'R'),full.names=T),
                    list.files(file.path(dir,packageName,'man'),full.names=T))
        rm('.foo',envir=env)
    }
    # just in case
    safe.dir.create(file.path(dir,packageName,'R'))
    safe.dir.create(file.path(dir,packageName,'man'))

    # kill the "read-and-delete-me"
    file.remove( file.path(dir,packageName,"Read-and-delete-me") )

    message("Creating Makefile...")
    # Makefile
    cat(sprintf(
'default_target: all
#=============================================================================
RCMD = `/usr/bin/which R`
RSCRIPT = `/usr/bin/which Rscript`
BUILD_DIR=%s
PACKAGE_NAME=%s
#=============================================================================
# Set environment variables for the build.
all: clean package
doc: 
	@echo "Making documentation..."
	$(RSCRIPT) -e "library(utils); library(methods); library(roxygen2); roxygenise(\'$(PACKAGE_NAME)\')"
# remove dir inst if empty (roxygen bug)
	@if [ "$(ls -A $(PACKAGE_NAME)/inst)"=="" ]; then \\
		rm -rf $(PACKAGE_NAME)/inst; \\
	fi

package: doc
	@if [ ! -d "$(BUILD_DIR)" ]; then \\
		mkdir $(BUILD_DIR); \\
	fi
	(cd $(BUILD_DIR); $(RCMD) CMD build ../$(PACKAGE_NAME);)

check: package
	$(RCMD) CMD check -o $(BUILD_DIR) $(PACKAGE_NAME);

install: package
	@(if [ "${R_LIBS_USER}" != "" ] ; then \\
		echo "** Installing library" ;\\
		$(RCMD) CMD REMOVE -l ${R_LIBS_USER} $(PACKAGE_NAME) ;\\
		$(RCMD) CMD INSTALL -l ${R_LIBS_USER} $(BUILD_DIR)/$(PACKAGE_NAME)*.tar.gz ;\\
	else \\
		echo "** Installing library as root" ;\\
		$(RCMD) CMD REMOVE $(PACKAGE_NAME) ;\\
		$(RCMD) CMD INSTALL $(BUILD_DIR)/$(PACKAGE_NAME)*.tar.gz ;\\
	fi)
dev-install: clean check install
clean:
	rm -f $(PACKAGE_NAME)/man/*.Rd;
	rm -rf $(BUILD_DIR)/*;
.PHONY: clean
', build.dir, packageName), file=file.path(dir,'Makefile'))

    # Readme.
    message("Creating Readme...")
cat(sprintf('
FOR THE USER 
============
Get a zip/tar file with the package already generated,
 instead of installing from source.
> install.packages(\"%s_1.0.tar.gz\",repos=NULL)

FOR THE DEVELOPER
=================
R files are in %s/R.
Roxygen is used to generate documentation: all Rd files are generated and get written over.
A Makefile is provided for building.

From the terminal/command line [need to be able to run "R CMD ...."]
 make
 make install

This essentially runs (from directory %s):
 (run in R to generate docs):
 library(roxygen2)
 roxygenise("../%s")
 (run in shell):
 R CMD roxygen -d %s
 cd bin
 R CMD build ../%s
 R CMD INSTALL %s*.tar.gz
 cd ..

If you\'re developing and want to check the package (ie R CMD check), do
 make check

FROM WITHIN R
=============
This may or may not work...

Go to R, and set working dir to this directory.
> install.packages("%s",repos=NULL,type="source")

There is a possibility the documentation will be out of sync with the R files
 as I regenerate them afresh every time I modify the package -- if you want to
 be sure that the documentation is fresh, then:

> library(roxygen2)
> roxygenise("%s")
> install.packages("%s",repos=NULL,type="source")

Then, if all goes well:
> library(%s)
', packageName, packageName, build.dir, packageName, packageName,
packageName, packageName, packageName, packageName, packageName,
packageName),file=file.path(dir,'README'))

    # set up hg
    if ( hg ) {
        message("Setting up mercurial repository...")
        # .hgignore
        cat('
syntax:glob

*.roxygen
*.Rcheck
*.Rd

bin/*', file=file.path(dir,'.hgignore'))

        # hg init
        HG <- getProgram(c('hg','hg.exe'))
        if ( is.null(HG) ) {
            message(sprintf(paste(
              "Couldn't find 'hg' executable in your path,",
              "go to '%s' and type 'hg init'",
              "to initialise the repository.",collapse=" "),
                            file.path(dir)))
        } else {
            wd <- getwd()
            setwd(dir)
            # init
            system(sprintf('%s init',HG[1]),intern=T)
            # add/remove
            system(sprintf('%s add',HG[1]),intern=T)
            # commit
            if ( hg.commit )
                system(sprintf('%s commit -m "Initial commit"',HG[1]),intern=T)
            setwd(wd)
        }
    }

    message(sprintf("Package created, have a look at %s/README",dir))
}, error = function(e) {
    unlink(dir,recursive=TRUE)
    cat(gettext(e))
})
}


#' reload a package.
#' @param lib name of package to reload, unquoted or quoted.
#' @param force should a package be detached even though other attached
#' packages depend on it?
#' @param load boolean, whether to load the package if it wasn't already
#' loaded in the first place.
#' @param quietly boolean, suppress all warnings/messages/outputs.
#' @family dev
#' @export
reload <- function(lib,force=TRUE,load=TRUE,quietly=TRUE)
{
    func <- identity
    if ( quietly )
        func <- muffle
    pkgName <- as.character(substitute(lib))
    tryCatch({
    func(detach(sprintf('package:%s',pkgName),
           character.only=TRUE,
           unload=TRUE,force=force))
    }, error=function(e)
        if ( !load )
            stop("Package ",pkgName," could not be reloaded;",
                 " perhaps it was not loaded in the first place?")
    )
    func(library(pkgName, character.only=TRUE))
}

#' muffles as much output as possible (except errors)
#' @param ... what to muffle
#' @examples
#' muffle(cat('hi'))
#' muffle(message('hi'))
#' muffle(warning('warning!'))
#' @export
#' @family dev
muffle <- function(...)
{
    invisible(capture.output(
              suppressPackageStartupMessages(
              suppressMessages(
              suppressWarnings(
              ...
              )))))
}
